<?php

namespace Drupal\widget_type\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Menu\MenuLinkTreeElement;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Interactive Components controller.
 */
class InteractiveComponents extends ControllerBase {

  /**
   * The menu link tree.
   *
   * @var \Drupal\Core\Menu\MenuLinkTreeElement
   */
  private $menuTree;

  /**
   * InteractiveComponents constructor.
   *
   * @param \Drupal\Core\Menu\MenuLinkTreeInterface $menu_link_tree
   *   The menu link tree service.
   */
  public function __construct(MenuLinkTreeInterface $menu_link_tree) {
    $parameters = new MenuTreeParameters();
    $parameters->setRoot('interactive_components');
    $parameters->setMaxDepth(1);
    $this->menuTree = current($menu_link_tree->load('admin', $parameters));
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('menu.link_tree'));
  }

  /**
   * Controller entry point.
   *
   * @return array
   *   The response render array.
   *
   * @SuppressWarnings(PHPMD.StaticAccess)
   */
  public function __invoke() {
    // Make sure to list all the items in the menu.
    $links = array_map(static function (MenuLinkTreeElement $element) {
      $menu_link = $element->link;
      return [
        'title' => $menu_link->getTitle(),
        'url' => $menu_link->getUrlObject(),
      ];
    }, $this->menuTree->subtree);
    return [
      'intro' => [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t(
          'Interactive components are widget instances that are placed in a page. You can create widget instances manually or you can create them directly in the page builder pages.'
        ),
      ],
      'docs' => [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t(
          'Read the 📑 @link 📑 to gain more insight about the widget architecture.',
          [
            '@link' => Link::fromTextAndUrl(
              $this->t('introduction article'),
              Url::fromUri('https://www.lullabot.com/articles/widget-registry-how-serve-reusable-interactive-content-pieces'
              )
            )->toString(),
          ]
        ),
      ],
      'types' => [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t(
          'Each widget instance is has a widget type, or widget definition. Widget definitions may be created outside of Drupal by ingesting them from the widget registry using the %link module. You can also create the widget definitions manually using the form.',
          [
            '%link' => Link::fromTextAndUrl(
              $this->t('Widget Ingestion'),
              Url::fromUri('https://www.drupal.org/project/widget_ingestion')
            )->toString(),
          ]
        ),
      ],
      'links' => [
        '#theme' => 'links',
        '#links' => $links,
      ],
    ];
  }

}
